module Grants
  class ClientGroup
    def self.grants
      {
        create: {
            name: 'Anlegen',
            scopes: {
                system_wide: {
                    name: 'Systemweit',
                    detailed_description: "keine Einschränkungen" ,
                    user_types: [::Manager],
                    default_for: [],
                    manager: -> ability do
                      ability.can :create, ::ClientGroup
                    end
                },
                own_institutions: {
                    name: 'in eigenen Einrichtungen',
                    detailed_description: '
                      Eine Klasse darf nur an Einrichtungen angelegt werden, an denen der eingeloggte Lehrer arbeitet.
                      D.h. Es darf keine Einrichtung gelistet oder verwendet werden, die nicht beim Lehrer unter Einrichtungen/Einsatzorte aufgeführt ist.
                      Für Verwalter bedeutet dies, das eine Klasse nur an Einrichtungen angelegt werden kann, die unterhalb der Einrichtung des Verwalters in der
                      Einrichtungshierarchie liegen. Ruft man die Einrichtung des Verwalters auf, stehen diese unter "Untergeordnete Einrichtungen". Beachte: Diese Einrichtungen
                      können wiederum untergeordnete Einrichtungen haben.
                      ',
                    user_types: [::Teacher],
                    default_for:  [::Teacher],
                    manager: -> ability, manager do
                      ability.can :create, ::ClientGroup, {
                          institution:{
                              all_parent_institutions: { parent_id: manager.institution_id }
                          }
                      }
                    end,
                    teacher: -> ability, teacher do
                      ability.can :create, ::ClientGroup, {
                          institution:{
                              teacher_institution_relations: {
                                  teacher_id: teacher.id
                              }
                          }
                      }
                    end
                },
                leaded_institutions: {
                    name: 'in administrierten Einrichtungen (Schuladmin)',
                    detailed_description: '
                      Eine Klasse darf nur an Einrichtungen angelegt werden, an denen der eingeloggte Lehrer als Admin arbeitet.
                      D.h. Es darf keine Einrichtung gelistet oder verwendet werden, die nicht beim Lehrer unter Einrichtungen/Einsatzorte aufgeführt ist UND an
                      der unter Einsatzart eine Adminposition ausgewählt ist.
                      ',
                    user_types: [::Teacher],
                    default_for:  [],
                    teacher: -> ability, teacher do
                      ability.can :create, ::ClientGroup, {
                          institution:{
                              teacher_institution_relations: {
                                  is_school_admin: true,
                                  teacher_id: teacher.id
                              }
                          }
                      }
                    end
                }
            }
        },
        read: {
            name: 'Anzeigen',
            scopes: {
                system_wide: {
                    name: 'Systemweit',
                    detailed_description: "keine Einschränkungen" ,
                    user_types: [::Manager],
                    default_for: [],
                    manager: -> ability do
                      ability.can :read, ::ClientGroup
                    end
                },
                own_institutions: {
                    name: 'in eigenen Einrichtungen',
                    detailed_description: '
                      Eine Klasse darf nur an Einrichtungen gelesen werden, an denen der eingeloggte Lehrer arbeitet.
                      D.h.  eine Klasse in einer Einrichtung, an der der Lehrer nicht arbeitet, darf nirgendwo im System auftauchen.
                      Für Verwalter bedeutet dies, die Klasse muss in der Einrichtungshierarchie unter der Einrichtung des Verwalters liegen, um auftauchen zu dürfen.
                      ',
                    user_types: [::Manager,::Teacher],
                    default_for:  [::Teacher],
                    manager: -> ability, manager do
                      ability.can :read, ::ClientGroup, {
                          institution:{
                            all_parent_institutions: { parent_id: manager.institution_id }
                          }
                      }
                    end,
                    teacher: -> ability, teacher do
                      ability.can :read, ::ClientGroup, {
                          institution:{
                            teacher_institution_relations: {
                                teacher_id: teacher.id
                            }
                          }
                      }
                    end
                },
            }
        },
        update: {
            name: 'Bearbeiten',
            scopes: {
                system_wide: {
                    name: 'Systemweit',
                    detailed_description: "keine Einschränkungen" ,
                    user_types: [::Manager],
                    default_for: [],
                    manager: -> ability do
                      ability.can :update, ::ClientGroup
                    end
                },
                own_institutions: {
                    name: 'in eigenen Einrichtungen',
                    detailed_description: '
                      Eine Klasse darf nur an Einrichtungen bearbeitet werden, an denen der eingeloggte Lehrer arbeitet.
                      D.h.  eine Klasse in einer Einrichtung, an der der Lehrer nicht arbeitet, darf erstens keine Bearbeitungs-Buttons angezeigt bekommen und zweitens
                      darf ein speichern nicht erfolgreich sein, sollte doch ein Button angezeigt werden.
                      Für Verwalter bedeutet dies, die Klasse muss in der Einrichtungshierarchie unter der Einrichtung des Verwalters liegen, um bearbeitet werden zu dürfen.
                      ',
                    user_types: [::Manager,::Teacher],
                    default_for:  [::Teacher],
                    manager: -> ability, manager do
                      ability.can :update, ::ClientGroup, {
                          institution:{
                            all_parent_institutions: { parent_id: manager.institution_id }
                          }
                      }
                    end,
                    teacher: -> ability, teacher do
                      ability.can :update, ::ClientGroup, {
                          institution:{
                            teacher_institution_relations: {
                                teacher_id: teacher.id
                            }
                          }
                      }
                    end
                },
            }
        },
        destroy: {
            name: 'Löschen',
            scopes: {
                system_wide: {
                    name: 'Systemweit',
                    detailed_description: "keine Einschränkungen" ,
                    user_types: [::Manager],
                    default_for: [],
                    manager: -> ability do
                      ability.can :destroy, ::ClientGroup
                    end
                },
                own_institutions: {
                    name: 'in eigenen Einrichtungen',
                    detailed_description: '
                      Eine Klasse darf nur an Einrichtungen gelöscht werden, an denen der eingeloggte Lehrer arbeitet.
                      D.h.  eine Klasse in einer Einrichtung, an der der Lehrer nicht arbeitet, darf erstens keine Löschen-Buttons angezeigt bekommen und zweitens
                      darf ein löschen nicht erfolgreich sein, sollte doch ein Button angezeigt werden.
                      Für Verwalter bedeutet dies, die Klasse muss in der Einrichtungshierarchie unter der Einrichtung des Verwalters liegen, um gelöscht werden zu dürfen.
                      ',
                    user_types: [::Manager,::Teacher],
                    default_for:  [::Teacher],
                    manager: -> ability, manager do
                      ability.can :destroy, ::ClientGroup, {
                          institution_hierarchies: {
                              parent_id: manager.institution_id
                          }
                      }
                    end,
                    teacher: -> ability, teacher do
                      ability.can :destroy, ::ClientGroup, {
                          institution:{
                            teacher_institution_relations: {
                                teacher_id: teacher.id
                            }
                          }
                      }
                    end
                },
                leaded_institutions: {
                    name: 'in administrierten Einrichtungen (Schuladmin)',
                    detailed_description: '
                      Eine Klasse darf nur an Einrichtungen gelöscht werden, an denen der eingeloggte Lehrer als Schuladmin arbeitet.
                       D.h.  eine Klasse in an einer Einrichtung, an der der Lehrer nicht in einer Adminposition arbeitet, darf erstens keine Löschen-Buttons angezeigt bekommen und zweitens
                      darf ein löschen nicht erfolgreich sein, sollte doch ein Button angezeigt werden.
                      ',
                    user_types: [::Teacher],
                    default_for:  [],
                    teacher: -> ability, teacher, current_user do
                      ability.can :destroy, ::ClientGroup, {
                          institution:{
                              teacher_institution_relations: {
                                  is_school_admin: true,
                                  teacher_id: teacher.id
                              }
                          }
                      }
                    end
                }
            }
        }

    }
end



  end
end