class AdvancementPlanBorrow < ActiveRecord::Base

  attr_accessor :days, :do_return
  before_validation :set_end_date_from_days
  after_find :set_days_from_date_range

  validates :from_date, :advancement_plan_id, :institution_id, :teacher_id, presence: true
  validates :days, presence: true, if: "to_date.blank?"
  validates :to_date, presence: true, if: "days.blank?"

  validates :to_date, chronological_order: {fields: [:from_date,:to_date]}
  validate :validate_on_create, on: :create
  validate :validate_on_update, on: :update

  belongs_to :advancement_plan
  belongs_to :institution
  belongs_to :teacher

  has_one :portfolio_item_advancement_plan,
          class_name: 'Portfolio::ItemAdvancementPlan',
          foreign_key: :advancement_plan_borrow_id,
          inverse_of: :advancement_plan_borrow

  # all teacher_institution_relations were the institution is the same as this borrows institution ( Needed by auth )
  has_many :teacher_institution_relations, foreign_key: :institution_id, primary_key: :institution_id

  # all institution hierarchies were the parent_id is parent institution of this borrows institution ( Needed by auth )
  has_many :institution_hierarchies, foreign_key: :child_id,  primary_key: :institution_id

  scope :between_dates, -> (range) { where('NOT ((from_date < ? AND to_date < ?) OR (from_date > ? AND to_date > ?))',range.first,range.first,range.last,range.last) }
  scope :only_not_returned, -> { where('returned IS NULL') }
  scope :today, ->  { where('NOT ((from_date < ? AND to_date < ?) OR (from_date > ? AND to_date > ?))',Date.today,Date.today,Date.today,Date.today) }


  def date_range
    if from_date.present? && to_date.present?
      (from_date..to_date).to_a
    else
      (Date.today()..Date.today()).to_a
    end
  end

  private
  def set_end_date_from_days
    if to_date.blank? && days.present? && from_date.present?
      self.to_date = from_date + ( days.to_i - 1 )
    end
  end

  def set_days_from_date_range
    self.days = (to_date - from_date) + 1
  end

  def validate_on_update
    if do_return
      self.returned = Time.now
    end
  end

  def validate_on_create
    return false unless from_date.present? && to_date.present?
    today = Date.today()
    stock = AdvancementPlanInstitutionStock.where(:institution_id => institution_id, :advancement_plan_id => advancement_plan_id).take
    borrows = self.class.only_not_returned.where(:institution_id => institution_id, :advancement_plan_id => advancement_plan_id).between_dates(date_range)
    date_range.each do |date|
      current_borrows = 0
      borrows.each do |borrow|
        if borrow.from_date <= date && borrow.to_date >= date
          if borrow.teacher_id == teacher_id
            errors.add(:base,"Sie haben am #{I18n.l(date, format: :short)} schon ein Exemplar geliehen." ) and return true
          end
          current_borrows += 1 if today <= date # only count future conflicts
        end
      end
      if stock && current_borrows >= stock.stock
        # add error
        errors.add(:base,"Am #{I18n.l(date, format: :short)} sind keine Exemplare mehr verfügbar." ) and return true
      end
    end
    return true
  end

end
