# Klienten (Schüler, Kinder): alle Personen, für die Lehr- und Fördermaßnahmen angewandt werden
class Client < ActiveRecord::Base

  ######################################################################################################################
  ### Enums
  ######################################################################################################################
  ENUM_GENDER_MALE = 0
  ENUM_GENDER_FEMALE = 1

  # CHECK LIKE: Client.genders[client.gender] == Client::ENUM_GENDER_MALE
  enum gender: { "männlich" => ENUM_GENDER_MALE, "weiblich" => ENUM_GENDER_FEMALE }

  ENUM_SCHOOL_ATTENDANCE_SINCE_ARTICLE_35_1 = 0
  ENUM_SCHOOL_ATTENDANCE_SINCE_ARTICLE_35_2 = 1

  # CHECK LIKE: Client.school_attendance_since_articles[client.school_attendance_since_article] == Client::ENUM_SCHOOL_ATTENDANCE_SINCE_ARTICLE_35_1
  enum school_attendance_since_article: { "§ 35 Abs. 1 SchulG" => ENUM_SCHOOL_ATTENDANCE_SINCE_ARTICLE_35_1, "§ 35 Abs. 2 SchulG" => ENUM_SCHOOL_ATTENDANCE_SINCE_ARTICLE_35_2 }

  # this is not used, as it is desired to actually delete records and not mark them in any way
  # acts_as_paranoid

  ######################################################################################################################
  ### Clients can be subject of the notification module
  ######################################################################################################################
  # That means: there<<<< can be system notifications for clients
  include SystemTaskReceivable
  # there is also an explicit foreign key on client in notifications, as clients are VERY important
  #has_many :notifications, dependent: :destroy # refs #192

  # def system_notifications
  #   ids = notifications.ids
  #   ids << running_portfolio.notifications.ids
  #   associated_portfolio_items.each do |pi|
  #     ids << pi.notifications.ids
  #   end
  #   client_supporter_relations.each do |csr|
  #     ids << csr.notifications.ids
  #   end
  #   ids.flatten!
  #   ::Notification.only_showable.where("id IN (?)", ids)
  # end

  ######################################################################################################################
  ### Clients can trigger create and delete of system notifications
  ######################################################################################################################
  # that means: certain callbacks on client can result in created or deleted notifications
  include Publishable

  ######################################################################################################################
  ### Address record of client.
  ######################################################################################################################
  belongs_to :address, autosave: true, validate: true
  delegate :lat,:lng, :street, :number,:zip,:city,:state,:country, :to => :address, allow_nil: true
  accepts_nested_attributes_for :address
  validates :address, presence: true


  ######################################################################################################################
  ### Carreer items ( Laufbahneinträge )
  ######################################################################################################################
    # Career items are automatically created before changes on certain client attributes [teacher_id portfolio_level institution_id client_group_id]  are saved.
  # The created items will automatically include the change.
  has_many :career_items, autosave: true, validate: true, dependent: :destroy

  # It is possible to add a career_item of the correct change_type when changing a client attribute.
  # This is necessary, when the comment of the career item must be filled by the user
  accepts_nested_attributes_for :career_items

  # these are manually created career items, before the client was created
  has_many :former_career_items, validate: true
  accepts_nested_attributes_for :former_career_items, allow_destroy: true

  # these are automatically created career items, only the effective date can be altered
  has_many :system_career_items, validate: true
  accepts_nested_attributes_for :system_career_items

  before_save do # ensure_update_career_when_attribute_changes
    new_career_items = career_items.to_a.select(&:new_record?)
    if attribute_changed?(:teacher_id) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_TEACHER}.blank?
      career_items.build(change_type: CareerItem::CHANGE_TYPE_TEACHER)
    end
    if attribute_changed?(:portfolio_level) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_LEVEL}.blank?
      career_items.build(change_type: CareerItem::CHANGE_TYPE_LEVEL)
    end
    if attribute_changed?(:institution_id) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_INSTITUTION}.blank?
      career_items.build(change_type: CareerItem::CHANGE_TYPE_INSTITUTION)
    end
    if attribute_changed?(:client_group_id) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_CLIENT_GROUP}.blank?
      career_items.build(change_type: CareerItem::CHANGE_TYPE_CLIENT_GROUP)
    end
  end

  before_save do #  ensure_not_career_without_attribute_changes
    new_career_items = career_items.to_a.select(&:new_record?)
    if !attribute_changed?(:teacher_id) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_TEACHER}.any?
      self.career_items = career_items - new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_TEACHER}
    end
    if !attribute_changed?(:portfolio_level) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_LEVEL}.any?
      self.career_items = career_items - new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_LEVEL}
    end
    if !attribute_changed?(:institution_id) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_INSTITUTION}.any?
      self.career_items = career_items - new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_INSTITUTION}
    end
    if !attribute_changed?(:client_group_id) && new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_CLIENT_GROUP}.any?
      self.career_items = career_items - new_career_items.select{|ci| ci.change_type == CareerItem::CHANGE_TYPE_CLIENT_GROUP}
    end
  end

  ######################################################################################################################
  ### Associations needed by authorization module
  ######################################################################################################################
  # all teacher_institution_relations were the institution is the same as the clients institution ( Needed by auth )
  has_many :teacher_institution_relations, foreign_key: :institution_id, primary_key: :institution_id
  # all institution hierarchies were the parent_id is parent institution of the clients institution ( Needed by auth )
  has_many :institution_hierarchies, foreign_key: :child_id,  primary_key: :institution_id

  ######################################################################################################################
  ### client groups ( Klassenzugehörigkeit  )
  ######################################################################################################################
  # The current semester client group. This should be checked whenever the client group is needed.
  belongs_to :client_group, autosave: false, validate: false
  scope :without_client_group, -> { where(client_group_id: nil).where(is_former_client: false) }

  # The upcoming semester client group. When semester changes, the client group is written from next_semester_client_group_id -> client_group_id
  # Do not use this when the current client_group is needed
  belongs_to :next_semester_client_group, class_name: 'ClientGroup', autosave: false, validate: false

  validate do # ensure the next semester client_group is from the next semester
    if next_semester_client_group_id.present?
      errors.add(:next_semester_client_group_id, 'Muss aus dem nächsten Schuljahr stammen') unless ClientGroup.next_semester.where(id: next_semester_client_group_id).any?
    end
  end


  ######################################################################################################################
  ### Gutachten für Schülerportfolio
  ######################################################################################################################

  belongs_to :expertise_manager, class_name: 'Manager', foreign_key: "expertise_manager_id"
  belongs_to :expertise_school_inspector, class_name: 'Manager', foreign_key: "expertise_school_inspector_id"
  belongs_to :expertise_institution, class_name: 'Institution', foreign_key: "expertise_institution_id"
  belongs_to :expertise_teacher, class_name: 'Teacher', foreign_key: "expertise_teacher_id"

  enum expertise_manager_result: { expertise_manager_result_empty: 0, expertise_manager_result_accepted: 1, expertise_manager_result_rejected: 2 }
  enum expertise_inspector_result: { expertise_inspector_result_empty: 0, expertise_inspector_result_accepted: 1, expertise_inspector_result_rejected: 2 }
  enum expertise_institution_result: { expertise_institution_result_empty: 0, expertise_institution_result_accepted: 1, expertise_institution_result_rejected: 2 }
  enum expertise_teacher_result: { expertise_teacher_result_empty: 0, expertise_teacher_result_created: 1, expertise_teacher_result_rejected: 2 }

  before_save do
    if expertise_institution_id.blank?
      self.expertise_teacher_id=nil
    end
  end

  def expertise_allow_level_raise?
    if expertise_manager.present?
      expertise_manager_result.to_sym  == :expertise_manager_result_accepted
    else
      true
    end
  end

  # :choose_manager,
  # :choose_inspector,
  # :choose_institution,
  # :choose_teacher,
  # :teacher_result,
  # :institution_result,
  # :inspector_result,
  # :manager_result,
  def expertise_show_input(tag)
    ary = []
    ary << :choose_manager
    ary << :choose_inspector if expertise_manager.present?
    ary << :choose_institution if expertise_school_inspector.present?
    ary << :choose_teacher if expertise_institution.present?
    ary << :teacher_result if expertise_teacher.present?
    ary << :institution_result if expertise_teacher_result.to_sym  != :expertise_teacher_result_empty
    ary << :inspector_result if expertise_institution_result.to_sym  != :expertise_institution_result_empty
    ary << :manager_result if expertise_inspector_result.to_sym  != :expertise_inspector_result_empty
    ary.include?(tag)
  end

  def available_expertise_managers
    if institution.present?
      Manager.where(institution_id: institution.all_parent_institutions.pluck(:parent_id)).all
    else
      []
    end
  end

  def available_expertise_school_inspectors
    if institution.present?
      Manager.where(institution_id: institution.all_parent_institutions.pluck(:parent_id)).where(is_school_inspector: true).all
    else
      []
    end
  end

  def available_expertise_institutions
    if expertise_manager.present?
      Institution.where(id: expertise_manager.institution.all_child_institutions.pluck(:child_id)).where.not(id: institution_id).all
    else
      []
    end
  end

  def available_expertise_teachers
    if expertise_institution.present?
      ids = TeacherInstitutionRelation.joins(:teacher_institution_category)
          .where(institution_id: expertise_institution.id).pluck(:teacher_id)
      Teacher.where(id: ids).all
    else
      []
    end
  end


  ######################################################################################################################
  ### educational_courses ( Bildungsgänge  )
  ######################################################################################################################

  # Aktuelle Klassenstufenzugehörigkeit des Schülers
  has_many :educational_course_rules, through: :client_group, :class_name => '::EducationalCourseRule'

  # Vergangene Klassenstufenzugehörigkeit des Schülers
  has_many :client_educational_course_histories, validate: true, dependent: :destroy, :class_name => 'ClientEducationalCourseHistory'

  before_save do
    if attribute_changed?(:client_group_id) && client_group_id_was.present?
      if client_group_last_change.present?
        histories = client_educational_course_histories.all.index_by(&:course_stage_name)
        previous_client_group = ClientGroup.includes(:educational_course_rules).references(:educational_course_rules).where(id: client_group_id_was).take!
        previous_client_group.educational_course_rules.each do |rule|
          histories[rule.course_stage_name] = client_educational_course_histories.build(course_stage_name: rule.course_stage_name, duration_days: 0) unless histories[rule.course_stage_name]
          history_entry = histories[rule.course_stage_name]
          history_entry.duration_days += (Time.now - client_group_last_change) / 1.day
          history_entry.save( validate: false )
        end
      end
      self.client_group_last_change = Time.now
    end
  end

  def violated_course_rules
    latest_rule_ids = EducationalCourseRule.group('course_stage_name,id').having('created_at = MAX(created_at)').ids
    latest_rules = EducationalCourseRule.where(id: latest_rule_ids).all.index_by(&:course_stage_name)
    client_educational_course_histories.each do |history|
      applicable_rule = latest_rules[history.course_stage_name]
      applicable_rule.client = self
      applicable_rule.spent_time_days = 0 if applicable_rule.spent_time_days.blank?
      applicable_rule.spent_time_days += history.duration_days
    end
    educational_course_rules.each do |rule|
      applicable_rule = latest_rules[rule.course_stage_name]
      applicable_rule.client = self
      applicable_rule.running_time_days = 0 if applicable_rule.running_time_days.blank?
      if client_group_last_change
        applicable_rule.running_time_days +=  (Time.now - client_group_last_change) / 1.day
      end
    end
    ret = []
    latest_rules.each do |k,r|
      ret << r if r.violated?
    end
    return ret
  end


  ######################################################################################################################
  ### Einrichtungszugehörigkeit
  ######################################################################################################################
  belongs_to :institution, autosave: false, validate: false

  scope :former_clients, -> { where( is_former_client: true ) }

  validates :institution_id, presence: true

  validate do # Ist Bildungseinrichtung zugeordnet worden?
    errors.add(:institution_id,'Nur Bildungseinrichtungen möglich') if (institution.present? && ![Institution::TYPE_SCHOOL,Institution::TYPE_EDUCATIONAL].include?(institution.institution_type))
  end


  ######################################################################################################################
  ###  special education | Unterstützung durch sonderpädagische Fachkraft
  ######################################################################################################################
  # optional, kann auch nil sein
  belongs_to :teacher, autosave: false, validate: false

  ######################################################################################################################
  ### Eltern / Erziehungsberechtigte / Vormünder
  ######################################################################################################################
  has_many :client_guardian_relations, inverse_of: :client, autosave: true, validate: true, dependent: :destroy  # refs #192
  has_many :guardians, through: :client_guardian_relations, inverse_of: :clients, autosave: true, validate: true, dependent: :destroy # refs #192

  # Eltern sollen nicht gespeichert werden, wenn der Benutzer kein Feld für den guardian ausgefüllt hat.
  accepts_nested_attributes_for :client_guardian_relations, allow_destroy: true, reject_if: proc { |attributes|
    # abweisen wenn: alle felder blank, außer gender und felder in unterformularen.
    attributes['guardian_attributes'].reduce(true){ |b, ary| b && (ary[0] == 'gender' || ary[0].end_with?('attributes') || ary[1].blank?) }
  }

  validate do
    unless client_guardian_relations.to_a.delete_if(&:marked_for_destruction?).any?
      error_message = "Es muss mindestens eine Bezugsperson angegeben werden."
      errors.add(:base, error_message)
      false
    end
  end

  validate do
    unless client_guardian_relations.select { |cgr| !cgr.marked_for_destruction? && cgr.has_custody == true }.any?
      error_message = "Es muss mindestens eine erziehungsberechtigte Person eingetragen werden (leibl. Eltern, Pflegeeltern oder sonstige Personen)."
      client_guardian_relations.each do |cgr|
        cgr.errors.add(:has_custody, error_message)
      end
      false
    end
  end

  def client_guardian_relations_father
    unless client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_FATHER}.any?
      client_guardian_relations.build(category: ClientGuardianRelation::ENUM_CATEGORY_FATHER, guardian: Guardian.new(gender: Guardian::ENUM_GENDER_MALE))
    end
    client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_FATHER}
  end

  def client_guardian_relations_mother
    unless client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_MOTHER}.any?
      client_guardian_relations.build(category: ClientGuardianRelation::ENUM_CATEGORY_MOTHER, guardian: Guardian.new(gender: Guardian::ENUM_GENDER_FEMALE))
    end
    client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_MOTHER}
  end

  def client_guardian_relations_foster_father
    unless  client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_FOSTER &&  c.guardian[:gender] == Guardian::ENUM_GENDER_MALE}.any?
      client_guardian_relations.build(category: ClientGuardianRelation::ENUM_CATEGORY_FOSTER, guardian: Guardian.new(gender: Guardian::ENUM_GENDER_MALE))
    end
    client_guardian_relations.select{|c| ( c[:category] == ClientGuardianRelation::ENUM_CATEGORY_FOSTER && c.guardian[:gender] == Guardian::ENUM_GENDER_MALE)}
  end

  def client_guardian_relations_foster_mother
    unless client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_FOSTER && c.guardian[:gender] == Guardian::ENUM_GENDER_FEMALE}.any?
      client_guardian_relations.build(category: ClientGuardianRelation::ENUM_CATEGORY_FOSTER, guardian: Guardian.new(gender: Guardian::ENUM_GENDER_FEMALE))
    end
    client_guardian_relations.select{|c| c[:category] == ClientGuardianRelation::ENUM_CATEGORY_FOSTER && c.guardian[:gender] == Guardian::ENUM_GENDER_FEMALE}
  end

  def client_guardian_relations_legal
    unless client_guardian_relations.select{|c| [ClientGuardianRelation::ENUM_CATEGORY_LEGAL,ClientGuardianRelation::ENUM_CATEGORY_CUSTODY].include?(c[:category]) }.any?
      client_guardian_relations.build(category: ClientGuardianRelation::ENUM_CATEGORY_LEGAL, has_custody: true, guardian: Guardian.new )
    end
    client_guardian_relations.select{|c| [ClientGuardianRelation::ENUM_CATEGORY_LEGAL,ClientGuardianRelation::ENUM_CATEGORY_CUSTODY].include?(c[:category])}
  end

  ######################################################################################################################
  ### Schülerportfolio
  ######################################################################################################################
  # Der Client hat ein attribut portfolio_level. Hier wird immer das aktuelle Level gespeichert.
  # Dies ist das aktive Schülerportfolio. Portfolioeinträge werden nicht zusammen mit dem Client erzeugt/geändert
  has_many :portfolio_items,
           through: :running_portfolio,
           autosave: false,
           validate: false

  has_many :portfolios, as: :subject,
           class_name: '::Portfolio::Portfolio',
           inverse_of:  :subject,
           autosave: true,
           dependent: :destroy

  belongs_to :running_portfolio,
             foreign_key: :running_portfolio_id,
             class_name: '::Portfolio::Portfolio'

  has_many :portfolio_dbrcs, dependent: :destroy, :class_name => 'Portfolio::Dbrc'

  before_create do
    if portfolios.blank?
      build_new_portfolio
    end
    true
  end

  after_create do
    if running_portfolio.blank?
      update_attribute(:running_portfolio_id, portfolios.last.id)
    end
    true
  end

  before_save do
    if portfolio_level_changed?
      running_portfolio.update_attribute(:portfolio_level,portfolio_level) unless running_portfolio.blank?
    end
    true
  end

  def create_new_portfolio
    build_new_portfolio
    self.running_portfolio = portfolios.last
    save
    running_portfolio
  end

  def build_new_portfolio
    portfolios.build({
       rule_set_id: ::SystemSetting.get_instance.load_latest_portfolio_rules,
       rule_set_date: ::Date.today,
       portfolio_level: portfolio_level
    })
  end

  # get all portfolio items where the client is involved
  # That means, portfolio items are returned that match one of the following:
  #
  # 1. items of clients that are subject of the portfolio item
  #
  # @return [ActiveRecordRelation<Portfolio::Item>] collection of portfolio items
  def associated_portfolio_items
    ::Portfolio::Item.joins(:portfolio).
        references(:portfolio).
        where('portfolios.subject_id = ? AND portfolios.subject_type=? OR portfolios.subject_id = ? AND portfolios.subject_type=?',
               id,'Client',client_group_id,'ClientGroup' )
  end



  def can_lower_portfolio_level?
    running_portfolio.can_lower_portfolio_level(portfolio_level)
  end

  def can_raise_portfolio_level?
    running_portfolio.can_raise_portfolio_level(portfolio_level) && expertise_allow_level_raise?
  end

  def has_closed_expertise?
    portfolio_items.each do |i|
      if i.portfolio_step.type=='expertise' && i.closed?
        return true
      end
    end
    return false
  end

  def raise_portfolio_level!
    _v = ( has_closed_expertise? ? 3 : running_portfolio.next_rule_level)
    update_attribute(:portfolio_level, _v)
  end

  def lower_portfolio_level!
    update_attribute(:portfolio_level,running_portfolio.last_rule_level)
  end

  # what level had the client at a given date?
  def portfolio_level_at(date)
    CareerItem.portfolio_level_at_date(self,date)
  end

  # what is the highest level the client had after a given date?
  def max_portfolio_level_after(date)
    CareerItem.max_portfolio_level_after_date(self,date)
  end


  ######################################################################################################################
  ### externe Personen (Psychologen,Ärzte)
  ######################################################################################################################
  has_many :client_supporter_relations, inverse_of: :client, dependent: :destroy # refs #192
  has_many :supporters, through: :client_supporter_relations, inverse_of: :clients


  ######################################################################################################################
  ### check on duplicate records
  ######################################################################################################################
  class << self
    # gender [:m,:f]
    # bday Date
    # first and last string
    # def get_possible_duplicates(first_name,last_name,gender,bday)
    #   matches = []
    #   return matches unless first_name.present? && last_name.present? && gender.present? && bday.present?
    #   where(gender: gender==:m ? ENUM_GENDER_MALE : ENUM_GENDER_FEMALE).where(bday: bday).find_each(batch_size: 500) do |c|
    #      if c.last_name.similar?(last_name) && c.first_name.similar?(first_name)
    #        matches << c
    #      end
    #   end
    #   return matches
    # end
  end
  #belongs_to :client_duplicate_request
  # A duplicate request has been performed
  # on create: when a client_duplicate_request record exists
  # on update: always
  # def duplicate_request_done
  #   return true if client_duplicate_request_id.present? && new_record?
  #   return true if persisted?
  #   return false
  # end
  # after_create do
  #   if client_duplicate_request_id.present?
  #     client_duplicate_request.destroy
  #   end
  # end

  ######################################################################################################################
  ### Client phonetic hashes
  ######################################################################################################################
  # new duplicate detetcion strategy, > vsb v0.15.0, csb v0.14.0

  has_one :phonetic, class_name: 'ClientPhonetic',
          foreign_key: :client_id,
          inverse_of: :client,
          dependent: :destroy,
          autosave: true

  # keep phonetics up to date
  before_save do
    build_phonetic unless  phonetic.present?
    phonetic.last_name_koelner_phonetic_hash = last_name.phonetic_code
    phonetic.first_name_koelner_phonetic_hash  = first_name.phonetic_code
    phonetic.last_name_double_metaphone_pri_hash = last_name.metaphone2[0]
    phonetic.last_name_double_metaphone_alt_hash  = last_name.metaphone2[1]
    phonetic.first_name_double_metaphone_pri_hash  = first_name.metaphone2[0]
    phonetic.first_name_double_metaphone_alt_hash =   first_name.metaphone2[1]
    # autosaved via has_one option
  end

  attr_accessor :similar_clients
  attr_accessor :dont_check_similar_clients
  before_save do
    unless dont_check_similar_clients || (persisted? && !first_name_changed? && !last_name_changed? && !bday_changed?)
      sc = []
      sc = Client.remote_duplicates(first_name,last_name,bday,id)
      if sc.any?
        self.similar_clients =  sc
        errors.add(:similar_clients, 'Der Schüler existiert vielleicht schon')
      end
    end
    errors.any? ? false : true
  end

  class << self

    def remote_duplicates(first_name,last_name,bday, exclude_id)
      sc = CsbApi::SimilarClient.where(client_first_name: first_name, client_last_name: last_name,client_bday: bday).all
      sc.delete_if{|dup| dup.attributes['similar-client-id'] == exclude_id.to_s && dup.attributes['vsb-id'] == SystemSetting.get_instance.vsb_id}
    end

    # provides a list of local duplicated for a client.
    def duplicates(first_name,last_name,bday)
      last_name_phonetic_code = last_name.phonetic_code
      first_name_phonetic_code = first_name.phonetic_code
      last_name_double_metaphone_pri = last_name.double_metaphone[0]
      first_name_double_metaphone_pri = first_name.double_metaphone[0]
      last_name_double_metaphone_alt = last_name.double_metaphone[1]
      first_name_double_metaphone_alt = first_name.double_metaphone[1]
      # firstname matches either koelner_phonetic OR double_metaphone AND
      # lastname matches either koelner_phonetic OR double_metaphone
#       where(bday: bday).joins(:phonetic).where(
#           "client_phonetics.first_name_koelner_phonetic_hash=? OR client_phonetics.first_name_double_metaphone_alt_hash=? OR
# client_phonetics.first_name_double_metaphone_pri_hash=?",
      where(bday: bday).joins(:phonetic).where(
          "(client_phonetics.last_name_koelner_phonetic_hash=? AND client_phonetics.first_name_koelner_phonetic_hash=?) OR
          ((client_phonetics.last_name_double_metaphone_pri_hash=? OR client_phonetics.last_name_double_metaphone_alt_hash=?)
AND (client_phonetics.first_name_double_metaphone_pri_hash=? OR client_phonetics.first_name_double_metaphone_alt_hash=?) ) OR
((client_phonetics.last_name_double_metaphone_pri_hash=? OR client_phonetics.last_name_double_metaphone_alt_hash=?)
AND (client_phonetics.first_name_double_metaphone_pri_hash=? OR client_phonetics.first_name_double_metaphone_alt_hash=?) )",
          last_name_phonetic_code, first_name_phonetic_code,
          last_name_double_metaphone_pri, last_name_double_metaphone_pri,
          first_name_double_metaphone_pri, first_name_double_metaphone_pri,
          last_name_double_metaphone_alt, last_name_double_metaphone_alt,
          first_name_double_metaphone_alt, first_name_double_metaphone_alt).all
    end
  end

  ######################################################################################################################
  ### Suggestions
  ######################################################################################################################
  # Schüler sollen Vorschläge für Literatur bekommen.

  def diagnostic_suggestions(limit=1)
    begin
      @diagnostic_suggestions = CsbApi::DiagnosticSuggestion.where(client_ids: id).all.map(&:build_model) # TODO: currently, 1 random entries is returned per client, limit ignored
    rescue JsonApiClient::Errors::ConnectionError
      @diagnostic_suggestions = []
    end
  end

  def advancement_plan_suggestions(limit=1)
    begin
      @diagnostic_suggestions = CsbApi::AdvancementPlanSuggestion.where(client_ids: id).all.map(&:build_model) # TODO: currently, 1 random entries is returned per client, limit ignored
    rescue JsonApiClient::Errors::ConnectionError
      @diagnostic_suggestions = []
    end
  end

  ######################################################################################################################
  ### Search
  ######################################################################################################################
  # Schüler sollen bei der Suche immer nur mit dem vollen Namen exakt verglichen werden
  before_save do # full name search field contains concatenation of name with only characters A-Z and a-z
    self.full_name_search = (first_name+last_name).gsub(/\s*/,'')
  end

  def full_name
    "#{first_name} #{last_name}"
  end

  ######################################################################################################################
  ### Einrichtungswechsel
  ######################################################################################################################
  before_save do
    if attribute_changed?(:institution_id) && persisted?
      self.client_group_id = nil
    end
  end

  ######################################################################################################################
  ### Validierungen
  ######################################################################################################################

  validates :first_name, :last_name, :gender, :bday, presence: true
  validates :school_attendance_since, :school_attendance_since_article, presence: true
  validates :nationality, :language, :denomination, presence: true
  validate do
      unless bday.nil?
        errors.add(:base, "Bitte ein Geburtsdatum angeben das in der Vergangenheit liegt.") and return false if bday > Date.today
      end
  end

  ######################################################################################################################
  ### Readonly Attribute
  ######################################################################################################################
  # aktuelles Schulbesuchsjahr (auf ganze Jahre gezählte Laufbahneinträge)
  # TODO später sind hier noch die Regeln zu den Phasen der Schulbesuchsjahre relevant
  def education_year
    ((DateTime.now() - self.school_attendance_since).to_i / 365) + 1
  end

  def current_age
    ((Date.today - bday).to_i / 365).to_i # here no plus one, 1 year old means exactly 1 year
  end


end
