module Portfolio
  class ItemEvaluation < ::ActiveRecord::Base
      self.table_name = 'portfolio_item_evaluations'
      self.inheritance_column = nil

      alias_attribute :subject_involved, :client_involved

      belongs_to :portfolio_item_target,
                 foreign_key: :portfolio_item_target_id,
                 class_name: '::Portfolio::ItemTarget',
                 inverse_of:  :evaluation
      delegate :portfolio_item, to: :portfolio_item_target
      delegate :portfolio_subject, to: :portfolio_item
      delegate :portfolio, to: :portfolio_item


      has_many   :evaluation_involved_persons,
                 foreign_key: :portfolio_item_evaluation_id,
                 class_name: '::Portfolio::EvaluationInvolvedPerson',
                 inverse_of:  :portfolio_item_evaluation,
                 dependent: :destroy # refs #192
      accepts_nested_attributes_for  :evaluation_involved_persons

      has_many   :evaluation_involved_supporters,
                 foreign_key: :portfolio_item_evaluation_id,
                 class_name: '::Portfolio::EvaluationInvolvedSupporter',
                 inverse_of:  :portfolio_item_evaluation,
                 dependent: :destroy # refs #192

      has_many   :involved_supporters,
                 through: :evaluation_involved_supporters,
                 class_name: '::Supporter',
                 source: :supporter,
                 :after_remove => :after_remove_involved_supporter

      has_many   :evaluation_involved_teachers,
                 foreign_key: :portfolio_item_evaluation_id,
                 class_name: '::Portfolio::EvaluationInvolvedTeacher',
                 inverse_of:  :portfolio_item_evaluation,
                 dependent: :destroy # refs #192

      has_many   :involved_teachers,
                 through: :evaluation_involved_teachers,
                 class_name: '::Teacher',
                 source: :teacher,
                 :after_remove => :after_remove_involved_teacher

      has_many   :evaluation_involved_guardians,
                 foreign_key: :portfolio_item_evaluation_id,
                 class_name: '::Portfolio::EvaluationInvolvedGuardian',
                 inverse_of:  :portfolio_item_evaluation,
                 dependent: :destroy # refs #192

      has_many   :involved_guardians,
                 through: :evaluation_involved_guardians,
                 class_name: '::Guardian',
                 source: :guardian,
                 :after_remove => :after_remove_involved_guardian

      # Direct behavior rating
      # ---------------------------------------------------------------------------------------------------
      belongs_to  :portfolio_item_evaluation_dbr,
                  class_name: '::Portfolio::ItemEvaluationDbr',
                  foreign_key: :portfolio_item_evaluation_dbr_id,
                  inverse_of:  :portfolio_item_evaluation,
                  dependent: :destroy
      accepts_nested_attributes_for :portfolio_item_evaluation_dbr, reject_if: :all_blank
      # ---------------------------------------------------------------------------------------------------



      # Eager Creation and destruction of results
      # See Portfolio::EvaluationInvolved<Guardian|Person|Supporter|Teacher> for other places of creation
      # See Portfolio::EvaluationInvolvedPerson for destruction of these
      # ---------------------------------------------------------------------------------------------------

      # results are destroyed, when the related record is removed
      def after_remove_involved_teacher(teacher)
        portfolio_item_target.results.where({ evaluative: teacher}).delete_all
        return true
      end
      def after_remove_involved_guardian(guardian)
        portfolio_item_target.results.where({ evaluative: guardian}).delete_all
        return true
      end
      def after_remove_involved_supporter(supporter)
        portfolio_item_target.results.where({ evaluative: supporter}).delete_all
        return true
      end

      # when the result scheme changes, destroy all results of the previous scheme for the involved people and create new results
      # when there are already not null results, the schema should not be changed any more, as it could accidently discard user input
      before_save do
        if result_scheme_changed? && self.persisted? #refs #291
          evaluation_involved_persons.each(&:handle_result_scheme_change!)
          evaluation_involved_supporters.each(&:handle_result_scheme_change!)
          evaluation_involved_teachers.each(&:handle_result_scheme_change!)
          evaluation_involved_guardians.each(&:handle_result_scheme_change!)
          # the portfolio_item_evaluation_dbr is NOT destoyed on purpose, as it does no harm and the user can
          # get back to the filled form by switching the theme, even after saving
        end
        true
      end

      # results for subject are created here, immediately when the evaluation is persisted.
      before_save do
        if subject_involved?
          if portfolio_item_target.results.where(evaluative: portfolio.subject).none?
            portfolio_item_target.results.create({result_scheme: result_scheme, evaluative: portfolio.subject})
          end
        else
          portfolio_item_target.results.where(evaluative: portfolio.subject).delete_all if portfolio_item_target.results.where(evaluative: portfolio.subject).any?
        end
        true
      end

      validate do
        if result_scheme_changed? && ! still_unfilled?
          errors.add(:result_scheme,"Es wurden schon Ergebnisse eingetragen, das Schema kann nicht mehr geändert werden.")
        end
      end
      # ---------------------------------------------------------------------------------------------------

      # get the value of an attribute for validation purposes
      #
      # this method is used by ActiveModel::Errors when error message translations are done
      # List any keys here, that are not part of the model attributes but need error messages.
      def read_attribute_for_validation(attribute)
        send(attribute) unless [:any_involvements_validation].include?(attribute)
      end

      before_validation  { |record| record.start_date = record.portfolio_item.present? ? record.portfolio_item.start : Date.today }

      validates :start_date, :end_date, date: true
      validates :end_date,:result_scheme, presence: true
      validates :end_date, chronological_order: { fields: [:start_date,:end_date]}
      validates :end_date, chronological_order: {fields: ['portfolio_item_target.portfolio_item.start',:end_date]}
      validates :result_scheme, inclusion: ::Portfolio::ItemTargetResult.result_schemes

      validate :validate_any_involvements
      validate :evaluation_end_date_after_any_measurement_start

      def any_involvements?
        !(involved_teachers.blank? && involved_guardians.blank? && evaluation_involved_persons.blank? && evaluation_involved_supporters.blank? && !subject_involved?)
      end

      def count_involvements
        involved_teachers.count + involved_guardians.count + evaluation_involved_persons.count +  evaluation_involved_supporters.count + (subject_involved? ? 1 : 0)
      end

      def still_unfilled?
        return true if portfolio_item_target.results.empty?
        return portfolio_item_target.results.inject(true) do |memo,result|
          memo & result.result_value.nil?
        end
      end

      def filled_out?
        return false if portfolio_item_target.results.empty?
        return portfolio_item_target.results.inject(true) do |memo,result|
          memo & ! result.result_value.nil?
        end
      end

      def successful?
        return false unless filled_out?
        portfolio_item_target.results.map(&:result_normalized).inject(:+) >= 0
      end

      # get the records that are allowed to be set for relation
      #
      # @return [Array<String>]
      def available_result_schemes
        portfolio_subject.evaluation_available_result_schemes(self)
      end


      #  get the records that are  allowed to be set as involved guardians
      #
      # this is used by the select field for evaluation involved teachers in portfolio item create/update form
      #
      # @return [ActiveRecord::Associations::CollectionProxy<::Guardian>]
      def available_involved_guardians
        portfolio_subject.evaluation_available_involved_guardians(self)
      end

      # get the records that are allowed to be set as involved teachers
      # which are teachers at the same institution as the client
      #
      # this is used by the select field for evaluation involved teachers  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Teacher>]
      def available_involved_teachers
        portfolio_subject.evaluation_available_involved_teachers(self)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related to the client
      #
      # this is used by the select field for evaluation involved supporters  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def available_involved_supporters
        portfolio_subject.evaluation_available_involved_supporters(self)
      end

      private
      def validate_any_involvements
        if !any_involvements?
          errors.add(:any_involvements_validation, :blank)
        end
      end

      def evaluation_end_date_after_any_measurement_start
        latest_start_date = nil
        portfolio_item_target.portfolio_item.measurements.each do |measurement|
          next if measurement.begin.nil?
          if latest_start_date.nil? || latest_start_date < measurement.begin
            latest_start_date = measurement.begin
          end
        end
        portfolio_item_target.portfolio_item.advancement_plans.each do |measurement|
          next if measurement.begin.nil?
          if latest_start_date.nil? || latest_start_date < measurement.begin
            latest_start_date = measurement.begin
          end
        end
        portfolio_item_target.portfolio_item.diagnostics.each do |measurement|
          next if measurement.begin.nil?
          if latest_start_date.nil? || latest_start_date < measurement.begin
            latest_start_date = measurement.begin
          end
        end
        return true if end_date.nil?
        unless latest_start_date.nil? || end_date >= latest_start_date
          errors.add(:end_date, "Bitte erst ab #{::I18n.l(latest_start_date, format: :short)}. Dann haben alle Methoden begonnen.")
        end
      end
    end
end


