module Portfolio
  module Subjects
    class Client < Base

      def name
        "#{@subject.first_name} #{@subject.last_name}"
      end

      def client_group
        @subject.client_group
      end

      def client_group_id
        @subject.client_group_id
      end

      def is_class_teacher?(teacher)
        @subject.client_group.present? && teacher.id == @subject.client_group.teacher_id
      end

      def is_special_ed?(teacher)
        teacher.id == @subject.teacher_id
      end

      def has_special_ed?
        @subject.teacher && @subject.teacher.teacher_category.is_special_ed?
      end

      def has_extern_special_ed?
        has_special_ed? && ! @subject.teacher.institutions.map(&:id).include?(@subject.institution_id)
      end

      def relation_description_for_guardian(guardian)
        rel = guardian.client_guardian_relations.where(client_id: @subject.id).take
        rel.category unless rel.blank?
      end

      # get the advancement_plans for diagnostics that are allowed/availiable for the portfolio item
      #
      # the advancement_plans are always fetched using the csb api. The returned advancement_plans may or may not
      # exist locally.
      #
      # @return [Array<AdvancementPlan>]
      def available_remote_advancement_plan_suggestions
        ::CsbApi::AdvancementPlanSuggestion.where(client_ids: @subject.id).all.map(&:build_model)
      end

      # get the suggestions for diagnostics that are allowed/availiable for the portfolio item
      #
      # the diagnostics are always fetched using the csb api. The returned diagnostics may or may not
      # exist locally.
      #
      # The returned records do not include the dependecies of the diagnostics, like characteristics, norms, etc.
      # @see #Diagnostic.build_from_remote for getting full populated Diagnostic
      #
      def available_remote_diagnostic_suggestions
        ::CsbApi::DiagnosticSuggestion.where(client_ids: @subject.id).all.map(&:build_model)
      end

      # get the records that are  allowed to be set as involved guardians for this portfolio
      #
      # for base items, guardians of clients at the same institution as the portfolio are returned
      # this is used for the select involved guardians field in portfolio item create/update form
      #
      # @return [ActiveRecord::Associations::CollectionProxy<::Guardian>]
      def portfolio_item_available_involved_guardians(portfolio_item)
        ::Guardian.where(id: @subject.guardians.ids + portfolio_item.involved_guardians.ids).distinct(:id)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related all clients in the same institution as the portfolio
      #
      # this is used by the select involved supporters field in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def portfolio_item_available_involved_supporters(portfolio_item)
        # alle die dem Schüler zugeordnet sind
        _ids = ::Supporter.joins(:client_supporter_relations).where('client_supporter_relations.client_id=?',portfolio_item.portfolio.subject.id).pluck(:id)
        # alle die schon ausgewählt sind
        _ids += portfolio_item.involved_supporters.ids
        if portfolio_item.author.class.name == 'Supporter'
          _ids << portfolio_item.author.id
        end
        return ::Supporter.where(id: _ids);
      end

      #  get the records that are  allowed to be set as involved guardians
      # which are guardians related to the client
      # this is used by the select field for evaluation involved teachers in portfolio item create/update form
      #
      # @return [ActiveRecord::Associations::CollectionProxy<::Guardian>]
      def evaluation_available_involved_guardians(portfolio_item_evaluation)
        _records = ::Guardian.joins(:client_guardian_relations)
                   .where('client_guardian_relations.client_id =? OR guardians.id IN (?)',portfolio_item_evaluation.portfolio.subject.id,portfolio_item_evaluation.involved_guardians.ids)
        return _records.distinct(:id)
      end

      # get the records that are allowed to be set as involved supporters
      # which are supporters related to the client
      #
      # this is used by the select field for evaluation involved supporters  in portfolio item create/update form
      # @return [ActiveRecord::Associations::CollectionProxy<::Supporter>]
      def evaluation_available_involved_supporters(portfolio_item_evaluation)
        # alle die dem Schüler zugeordnet sind
        _ids = ::Supporter.joins(:client_supporter_relations).where('client_supporter_relations.client_id=?',portfolio_item_evaluation.portfolio.subject.id).pluck(:id)
        # alle die schon ausgewählt sind
        _ids += portfolio_item_evaluation.involved_supporters.ids
        if portfolio_item_evaluation.portfolio_item.author.class.name == 'Supporter'
          _ids << portfolio_item_evaluation.portfolio_item.author.id
        end
        return ::Supporter.where(id: _ids);
      end

      def subject_path
        routes = Rails.application.routes.url_helpers
        routes.dashboard_client_path(@subject)
      end

      def resolve_notifications_path
        routes = Rails.application.routes.url_helpers
        return routes.select_portfolio_item_date_clients_path(client_id: @subject.id)
      end

      def select_portfolio_item_date_path
        routes = Rails.application.routes.url_helpers
        routes.select_portfolio_item_date_clients_path(client_id: @subject.id)
      end

      def redirect_after_item_create
        routes = Rails.application.routes.url_helpers
        routes.dashboard_client_path(@subject)
      end

      def portfolio_title
        'Schülerportfolio'
      end

    end
  end
end