class User < ActiveRecord::Base
  belongs_to :identifiable, polymorphic: true
  include AuthorizationUserConcern

  # Include default devise modules. Others available are:
  # :confirmable, :lockable, :timeoutable and :omniauthable
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable

  validates_uniqueness_of :username, allow_blank: true
  validates_length_of :username, minimum: 5, maximum: 100, allow_blank: true
  before_save {self.username=nil if username.blank?}



  # this overridden function seems to be unnecessary, but without it, no email is sent by devise.
  # addtionally, sending by the message queue does not work, the smtp configuration that is used seems to
  # be the default.
  # TODO: find out what is the problem and fix it.
  def send_devise_notification(notification, *args)
    message = devise_mailer.send(notification, self, *args)
    message.deliver_now
  end

  before_create do
    self.super_admin=true unless User.any?
  end

  # Virtual attribute for authenticating by either username or email
  # This is in addition to a real persisted field like 'username'
  attr_accessor :login

  # called by devise to fetch user for given authentication conditions.
  # Behavior is as follows:
  # when key :login is given
  # -> use :login to query for username if that is not null, query for email otherwise
  # when key :username is given instead
  # -> use :username to query for username
  # when key :email is given instead
  # -> query for email
  def self.find_for_database_authentication(warden_conditions)
    conditions = warden_conditions.dup
    # in rails 4.x, to_h probably is empty and to_hash must be used instead
    conditions_h = conditions.to_h
    if conditions_h.empty?
      conditions_h = conditions.to_hash
    end
    if login = conditions.delete(:login)
      where(conditions_h).where(["(username is null AND email = :value) OR (username = :value)", { :value => login }]).first
    elsif conditions.has_key?(:username)
      where(conditions_h).where(["username = :value", { :value => login }]).first
    elsif conditions.has_key?(:email)
      where(conditions_h).where(["email = :value", { :value => login }]).first
    end
  end

  # Attempt to find a user by searching for the configured reset_password_keys in the order they are defined.
  # If a record is found, send new password instructions to it.
  # If none of these keys is found, returns a new user with an email not found error.
  # Attributes must contain one of the defined keys
  def self.send_reset_password_instructions(attributes={})
    recoverable = new
    reset_password_keys.each do |key|
      recoverable = find_or_initialize_with_errors([key], attributes, :not_found)
      break if recoverable.persisted?
    end
    recoverable.send_reset_password_instructions if recoverable.persisted?
    recoverable
  end

  def generate_reset_passwort_token
    set_reset_password_token
    #edit_password_url(@resource, reset_password_token: @token)
  end

  def person
    @person = identifiable unless @person.present?
    return @person
  end

  def username_display
    username || email
  end

  def user_category_display
    if is_teacher?
      return person.teacher_category.name
    else
      return person.class.name.humanize
    end
  end


  def is_teacher?
    self.identifiable.is_a? Teacher
  end

  def is_guardian?
    self.identifiable.is_a? Guardian
  end

  def is_supporter?
    self.identifiable.is_a? Supporter
  end

  def is_client?
    self.identifiable.is_a? Client
  end

  def is_manager?
    self.identifiable.is_a? Manager
  end

end
