#!/bin/bash
#-----------------------------------------------------------------------------------------------------------------------
#
# Install Vschoolboard Application Server
#
# USAGE: install.sh
#
#   Mandatory options:
#
#   --group=<string>
#     the webserver acting as proxy runs as this group
#   --user=<string>
#     the application server will run as this user
#   --email=<admin email>
#     login to webinterface with this email.
#     defaults to 'admin@localhost'
#
#   Other options:
#   --pass=<admin user password>
#     login to webinterface with user 'admin' and this password.
#     defaults to 'admin'
#   --region=<state/region>
#     a region that is listed in csb regions
#     defaults to 'NRW'
#   --name=<name>
#     name of this installation
#     defaults to 'Virtuelles Schulboard DEVELOPMENT'
#   --non-interactive
#     Defaults to 0
#   --env=<string>
#     can be production or development
#     Defaults to 'production'
#-----------------------------------------------------------------------------------------------------------------------

# This is used to access single values in application config file. Only useful for scalar values, because otherwise
# a string representation of the ruby datatype will be returned, which is quite useless here.
# usage some_value=$(get_config_value 'yaml_file' 'environment' 'value_name')
function get_config_value {
  local config_file=""
  local data_path=""
  local result=""
  while [ $# -gt 0 ]; do
    if [[ -z $config_file ]]; then
      config_file=$1
    else
      data_path="$data_path['$1']"
    fi
    shift
  done
  if [[ -f $config_file ]]; then
    cat $config_file | ruby -ryaml -e "data = YAML::load(STDIN.read); if(data.is_a? Hash); puts data$data_path; end"
    return 0
  fi
  return 1
}


if [ "$(id -u)" != "0" ]; then
   echo "This script must be run as root" 1>&2
   exit 1
fi

command -v bundler >/dev/null 2>&1 || { echo "I require bundler but it's not installed. Is ruby installed? Aborting." >&2; exit 1; }
command -v psql >/dev/null 2>&1 || { echo "I require psql but it's not installed. Is postgresql installed? Aborting." >&2; exit 1; }
command -v ruby >/dev/null 2>&1 || { echo "I require ruby but it's not installed. Aborting." >&2; exit 1; }
command -v crontab >/dev/null 2>&1 || { echo "I require crontab but it's not installed. Aborting." >&2; exit 1; }


# Just for information
APP_INSTANZ="Vschoolboard Application Server"
# current scriptname
SCRIPT=$(readlink -f "$0")
# folder of current script
SCRIPTPATH=$(dirname "$SCRIPT")
APP_VERSION_FILE="$SCRIPTPATH/version"
APP_CONFIG_FILE="$SCRIPTPATH/config/local_env.yml"
#APP_INSTALL_PATH=$(realpath "$SCRIPTPATH/../vsb")
APP_TMP_PATH="$SCRIPTPATH/tmp"
#APP_TMP_PATH="$APP_INSTALL_PATH/tmp"
APP_DATA_PATH=$(realpath "$SCRIPTPATH/../data")
APP_ENVIRONMENT="production"
APP_ADMIN_PASSWORD="vschoolboard"
APP_ADMIN_EMAIL="admin@localhost"
APP_REGION='NRW'
APP_NAME='Virtuelles Schulboard DEVELOPMENT'
INTERACTIVE=1
while [ $# -gt 0 ]; do
  case "$1" in
    --pass=*)
      APP_ADMIN_PASSWORD="${1#*=}"
      ;;
    --email=*)
      APP_ADMIN_EMAIL="${1#*=}"
      ;;
    --region=*)
      APP_REGION="${1#*=}"
      ;;
    --name=*)
      APP_NAME="${1#*=}"
      ;;
    --non-interactive*)
      INTERACTIVE=0
      ;;
    --env=*)
      APP_ENVIRONMENT="${1#*=}"
      ;;
    --group=*)
      APP_GROUP="${1#*=}"
      ;;
    --user=*)
      APP_USER="${1#*=}"
      ;;
    *)
      cat <<EOF
************************************************************************************************************************
USAGE: install.sh <options>

   Mandatory options:

   --group=<string>
     the webserver acting as proxy runs as this group
   --user=<string>
     the application server will run as this user
   --email=<admin email>
     login to webinterface with this email.
     defaults to 'admin@localhost'

   Other options:
   --pass=<admin user password>
     login to webinterface with this password.
     defaults to 'admin'
   --region=<state/region>
     a region that is listed in csb regions
     defaults to 'NRW'
   --name=<name>
     name of this installation
     defaults to 'Virtuelles Schulboard DEVELOPMENT'
   --non-interactive=1|0
     Interactive mode
     Defaults to 0
   --env=<string>
     can be production or development
     Defaults to 'production'

************************************************************************************************************************
EOF
      exit 1
  esac
  shift
done
if [ ${APP_ENVIRONMENT} == 'development' ]; then
  command -v git >/dev/null 2>&1 || { echo "I require git but it's not installed. Aborting." >&2; exit 1; }
  APP_USER=$(who | awk '{print $1}')
  APP_GROUP=$(id -gn $APP_USER)
fi
if [[ ! -v APP_GROUP ]]; then
  echo "Please set --group=<proxyserver group> or install for current user in development environment with --env=development" 1>&2
  exit 1
fi
if [[ ! -v APP_USER ]]; then
  echo "Please set --user=<applicationserver user>  or install for current user in development environment with --env=development" 1>&2
  exit 1
fi
# check local env is there
if [ ! -f ${APP_CONFIG_FILE} ] ; then
  echo "Please copy config/local_env.tmpl.yml to config/local_env.yml, adjust your configuration and run again!"
  exit 0
fi


DB_NAME=$(get_config_value ${APP_CONFIG_FILE} ${APP_ENVIRONMENT} VSCHOOLBOARD_DATABASE_DB_NAME)
DB_USER=$(get_config_value ${APP_CONFIG_FILE} ${APP_ENVIRONMENT} VSCHOOLBOARD_DATABASE_USER)
DB_PASS=$(get_config_value ${APP_CONFIG_FILE} ${APP_ENVIRONMENT} VSCHOOLBOARD_DATABASE_PASSWORD)
DB_HOST=$(get_config_value ${APP_CONFIG_FILE} ${APP_ENVIRONMENT} VSCHOOLBOARD_DATABASE_HOST)

if [[ -z ${DB_NAME} || -z ${DB_USER} || -z ${DB_PASS} || -z ${DB_HOST} ]]; then
  echo "Database name, password and/or role not found in configuration config/local_env.yml"
  exit 0
fi



echo "--------------------------------------------------------"
echo "Begin install:                              $APP_INSTANZ"
echo "Install environment is:                     $APP_ENVIRONMENT"
echo "Database name is:                           $DB_NAME"
echo "DB Access granted to postgres role:         $DB_USER"
echo "DB Access with password (hidden):           $(printf "%0.s*" $(seq 1 ${#DB_PASS}))"
echo "DB Host:                                    $DB_HOST"
if [ ! ${APP_ENVIRONMENT} == 'development' ]; then
  echo "Install Version:                            $(cat $APP_VERSION_FILE)"
  echo "User data will be placed in:                $APP_DATA_PATH"
fi
echo "--------------------------------------------------------"
if [[ ${INTERACTIVE} == 1 ]]; then
  echo -n "Proceed with this configuration? (y/n)"
  read answer
  if ! echo "$answer" | grep -iq "^y" ; then
    exit 0
  fi
fi

if [[ ! ${APP_ENVIRONMENT} == 'development' ]]; then
  # if this dir is not already 'vsb'
  if [[ ! ${PWD##*/} == 'vsb' ]]; then
    # if there is already a directory 'vsb' in the same folder
    if [[ -d "${PWD%/*}/vsb" ]]; then
      echo "There is already a vsb directory in this folder"
    fi
    mv "$PWD" "${PWD%/*}/vsb" # rename the source directory to vsb
    echo "Renamed src to vsb"
  fi
fi
APP_INSTALL_PATH=$(pwd -P)
# create directories
if [[ ${APP_ENVIRONMENT} == 'development' ]]; then
  mkdir -p "$APP_INSTALL_PATH/public/uploads";
else
  if [[ ! -d "$APP_DATA_PATH/uploads" ]] ; then
    mkdir -p "$APP_DATA_PATH/uploads";
    echo "Created application data directory $APP_DATA_PATH"
    ln -s "$APP_DATA_PATH/uploads" "$APP_INSTALL_PATH/public/uploads"
    echo "Linked application data directory into public folder. "
  fi
fi
mkdir -p "$APP_INSTALL_PATH/log"
mkdir -p "$APP_INSTALL_PATH/tmp/pids"

if [ -f "/etc/logrotate.conf" ] ; then
  if ! grep -iqE "$APP_INSTALL_PATH/log/" "/etc/logrotate.conf"; then
    if [[ ${INTERACTIVE} == 1 ]]; then
      echo -n "Append log rotation to /etc/logrotate.conf? (y/n)"
      read answer
      if echo "$answer" | grep -iq "^y" ; then
        do_append_to_logrotation=1
      else
        do_append_to_logrotation=0
      fi
    else
      do_append_to_logrotation=1
    fi
    if [[ ${do_append_to_logrotation} == 1 ]]; then
      echo "Appending logrotation to /etc/logrotate.conf "
      cat <<EOF >> /etc/logrotate.conf
"$APP_INSTALL_PATH/vsb/log/*.stderr" "$APP_INSTALL_PATH/vsb/log/*.stdout" "$APP_INSTALL_PATH/vsb/log/*.log" {
  daily
  missingok
  rotate 7
  compress
  delaycompress
  notifempty
  copytruncate
}    
EOF
    fi
  fi
fi


# copy configuration templates
if [ ! -f "$APP_INSTALL_PATH/config/environments/$APP_ENVIRONMENT.rb" ] ; then
  cp "$APP_INSTALL_PATH/config/environments/$APP_ENVIRONMENT.tmpl.rb"  "$APP_INSTALL_PATH/config/environments/$APP_ENVIRONMENT.rb"
  echo  "Created app configuration $APP_ENVIRONMENT.rb"
fi
if [ ! -f "$APP_INSTALL_PATH/config/puma/$APP_ENVIRONMENT.rb" ] ; then
  cp "$APP_INSTALL_PATH/config/puma/$APP_ENVIRONMENT.tmpl.rb"  "$APP_INSTALL_PATH/config/puma/$APP_ENVIRONMENT.rb"
  echo  "Created puma configuration $APP_ENVIRONMENT.rb"
fi
if [[ ! -f "$APP_INSTALL_PATH/config/database.yml" ]] ; then
  cp "$APP_INSTALL_PATH/config/database.tmpl.yml"  "$APP_INSTALL_PATH/config/database.yml"
  echo  "Created database configuration database.yml"
fi


# directory access rights
if [[ ! ${APP_ENVIRONMENT} == 'development' ]]; then
  chown -R "$APP_USER:$APP_GROUP" "$APP_INSTALL_PATH/../"
  echo "Recursive change owner to $APP_USER:$APP_GROUP"
  echo "-> $(realpath "$APP_INSTALL_PATH/../")"
  chmod -R 750  "$APP_INSTALL_PATH/../"
  echo "Set access rights to 750 for "
  echo "-> $(realpath "$APP_INSTALL_PATH/../")"
  chmod -R 770 "$APP_DATA_PATH"
  chmod -R 770 "$APP_INSTALL_PATH/log"
  chmod -R 770 "$APP_INSTALL_PATH/tmp"
  echo "Set access rights to 770 for "
  echo "-> $APP_DATA_PATH"
  echo "-> $APP_INSTALL_PATH/log"
  echo "-> $APP_INSTALL_PATH/tmp"
fi



bundler_executable="RAILS_ENV=${APP_ENVIRONMENT} $( command -v bundle )"
if [[ ${APP_ENVIRONMENT} == 'development' ]]; then
  echo "Installing gems as user=$APP_USER"
  echo "gems will be installed global."
  sudo -u ${APP_USER} ${bundler_executable} install
  echo "Gems installed."
else
  if [[ ! -d "$APP_INSTALL_PATH/vendor/bundle/" ]]; then
    echo "Installing gems as user=$APP_USER"
    echo "gems are installed in application vendor directory."
    sudo -u ${APP_USER} ${bundler_executable} install --deployment
    chown -R "$APP_USER:$APP_GROUP" "$APP_INSTALL_PATH/vendor/bundle"
    chmod -R 750  "$APP_INSTALL_PATH/vendor/bundle"
    echo "Gems installed."
  fi
fi


cur_dir=$(pwd)
cd /home
# create database
if [[ $DB_HOST != "localhost" && $DB_HOST != "127.0.0.1" ]]; then
 db_host_param=" -h $DB_HOST "
else
 db_host_param=""
fi
db_role_existing=$(sudo -u postgres psql ${db_host_param} -tAc "SELECT 1 FROM pg_roles WHERE rolname='$DB_USER'")
if [[ ${db_role_existing} != 1 ]]; then
  sudo -u postgres psql ${db_host_param} -c "CREATE USER $DB_USER WITH PASSWORD '$DB_PASS';"
  useradd -M -s /bin/false $DB_USER
  echo  "Created pg_role $DB_USER and system user $DB_USER with provided password"
fi
db_name_existing=$(sudo -u postgres psql ${db_host_param} -tAc "SELECT 1 FROM pg_database WHERE datname='$DB_NAME'")
if [[ ${db_name_existing} != 1 ]]; then
  sudo -u postgres psql ${db_host_param} -c "CREATE DATABASE $DB_NAME WITH OWNER '$DB_USER' ENCODING 'utf8'"
  echo "Created database $DB_NAME for pg_role=$DB_USER"
fi
db_is_initialized=$(sudo -u ${DB_USER} psql ${db_host_param} -d ${DB_NAME} -tAc "SELECT 1 WHERE to_regclass('public.schema_migrations') IS NOT NULL")
if [[ ${db_is_initialized} != 1 ]]; then
  if [[ ! -f ${APP_INSTALL_PATH}/db/structure.sql ]]; then
    echo "could not load schema. Abort. Make sure that ${APP_INSTALL_PATH}/db/structure.sql contains the schema."
    exit 1
  fi
  echo "Initializing database..."
  # COMMENT ON EXTENSION is commented out on the fly, because it generates an errors message that can be ignored.
  # CREATE SEQUENCE is altered on the fly, to ensure compatibility with postgres 9.x
  # the rake task db:structure:load isnt used here on purpose, as it executes the structure.sql TWICE
  if ! cat ${APP_INSTALL_PATH}/db/structure.sql | sed -E -e ':a' -e 'N' -e '$!ba' -e 's/CREATE SEQUENCE\s+(\S+)\s+AS\s+\S+\s+/CREATE SEQUENCE \1 /g' | sed -e 's/COMMENT ON EXTENSION/-- COMMENT ON EXTENSION/g' | sudo -u ${DB_USER} psql  ${db_host_param} -d ${DB_NAME} -q -f -; then
    echo "could not load schema. Abort. Make sure that ${APP_INSTALL_PATH}/db/structure.sql contains the schema."
    exit 1
  fi
  echo "Initialized database."
fi
cd ${APP_INSTALL_PATH}
if [[ ! -f ${APP_TMP_PATH}/db_seeded ]]; then
  echo "Loading seeds..."
  if sudo -u ${APP_USER} ${bundler_executable} exec rake db:seed; then
    sudo -u ${APP_USER} touch ${APP_TMP_PATH}/db_seeded
    echo "Default data loaded"
  else
    echo "Could not load default data"
    exit 1
  fi
fi
if [[ ! -f ${APP_TMP_PATH}/auth_initialized ]]; then
  echo "Initializing authorization module..."
  if sudo -u ${APP_USER} ${bundler_executable} exec rake vsb:sync_authorization_items > /dev/null; then
    sudo -u ${APP_USER} touch ${APP_TMP_PATH}/auth_initialized
    echo "Authorization module initialized"
  else
    echo "Authorization module could not initialize. Abort."
    exit 1
  fi
fi
if [[ ! ${APP_ENVIRONMENT} == 'development' ]]; then
  if ! sudo -u ${APP_USER} ${bundler_executable} exec rake assets:check > /dev/null 2>&1; then
    echo "Compiling assets..."
    if sudo -u ${APP_USER} ${bundler_executable} exec rake assets:precompile; then
       echo "Compiled assets."
    else
      echo "Asset compilation failed. Abort."
      exit 1
    fi
  fi
fi
# install cron
if ! sudo -u ${APP_USER} crontab -l > /dev/null 2>&1; then
  # user has no crontab
  current_crontab="# m h dom mon dow command"
else
  current_crontab=$(sudo -u ${APP_USER} crontab -l)
fi

if ! echo "$current_crontab" | grep -F "cd $APP_INSTALL_PATH && $bundler_executable exec rake vsb:cron" > /dev/null; then
    if [[ ${INTERACTIVE} == 1 ]]; then
      echo -n "Append schoolboard cron task to ${APP_USER} crontab? (y/n)"
      read answer
      if echo "$answer" | grep -iq "^y" ; then
        do_append_to_crontab=1
      else
        do_append_to_crontab=0
      fi
    else
      do_append_to_crontab=1
    fi
    if [[ $do_append_to_crontab == 1 ]]; then
      (
      echo "$current_crontab"
      echo "0 0 * * * cd $APP_INSTALL_PATH && $bundler_executable exec rake vsb:cron['daily']"
      echo "0 0 * * 1 cd $APP_INSTALL_PATH && $bundler_executable exec rake vsb:cron['weekly']"
      echo "0 0 1 * * cd $APP_INSTALL_PATH && $bundler_executable exec rake vsb:cron['monthly']"
      echo "0 0 * 1 * cd $APP_INSTALL_PATH && $bundler_executable exec rake vsb:cron['annually']"
      echo
      ) | sudo -u ${APP_USER} crontab -
      echo "Crontab entry for $APP_USER done."
    fi
fi

echo "Create System setup..."
if sudo -u ${APP_USER} ${bundler_executable} exec rake vsb:setup["${APP_ADMIN_PASSWORD}","${APP_REGION}","${APP_NAME}","${APP_ADMIN_EMAIL}"]; then
   echo "Setup completed."
else
  echo "Setup failed. Abort."
  exit 1
fi
sudo -u ${APP_USER} RAILS_ENV=${APP_ENVIRONMENT}  $( command -v bundle ) exec rake vsb:update_version

if [[ ! ${APP_ENVIRONMENT} == 'development' ]]; then
  echo "Installation completed. The application is not yet ready, further steps are described here:"
  echo "$SCRIPTPATH/README"
else
  echo "Development environment installed. Run srvctl.sh start development."
  echo "Open http://localhost:3000 (unless you configured sth. else)"
fi
exit 0
